// Arduino sketch for simple dash/sli using TFT LCD + NeoPixel strip with GameDash/SimTools
// Version 0.1 by TOPMO3
// Supports parameters below:
// S - speed, km/h
// G - gear
// R - RPM
// F - Fuel, % * 10
// O - Oil, C * 10
// W - Water, C * 10 
// U - Fuel, L * 10
// L - Lap
// X - Telemetry driven RPM LED blink, % of shift point (iRacing ShiftIndicatorPct)
// E - iRacing Engine Warnings
//
// Examples:
//
// LFS simple setup (only for 1st screen)
// R<Dash2>G<Dash3>S<Dash1>F<Dash6>
//
// iRacing setup
// S<Dash1>R<Dash2>G<Dash3>U<Dash5>L<Dash6>O<Dash7>X<Dash12>W<Dash13>E<Dash14>F<Dash4>
 
#include <Adafruit_GFX.h>    // Core graphics library
#include <Adafruit_TFTLCD.h> // Hardware-specific library

#include <EEPROM.h>

#include <Adafruit_NeoPixel.h>  //NeoPixel strip library
#define NeoPixelPIN 10                  //Strip wired to Pin 10
Adafruit_NeoPixel strip = Adafruit_NeoPixel(8, NeoPixelPIN, NEO_GRB + NEO_KHZ800);

// The control pins for the LCD can be assigned to any digital or
// analog pins...but we'll use the analog pins as this allows us to
// double up the pins with the touch screen (see the TFT paint example).
#define LCD_CS A3 // Chip Select goes to Analog 3
#define LCD_CD A2 // Command/Data goes to Analog 2
#define LCD_WR A1 // LCD Write goes to Analog 1+
#define LCD_RD A0 // LCD Read goes to Analog 0

#define LCD_RESET A4 // Can alternately just connect to Arduino's reset pin

#define FONT_START 0
#define FONT_END 1

// Assign human-readable names to some common 16-bit color values:
#define Black           0x0000      /*   0,   0,   0 */
#define Navy            0x000F      /*   0,   0, 128 */
#define DarkGreen       0x03E0      /*   0, 128,   0 */
#define DarkCyan        0x03EF      /*   0, 128, 128 */
#define Maroon          0x7800      /* 128,   0,   0 */
#define Purple          0x780F      /* 128,   0, 128 */
#define Olive           0x7BE0      /* 128, 128,   0 */
#define LightGrey       0xC618      /* 192, 192, 192 */
#define DarkGrey        0x7BEF      /* 128, 128, 128 */
#define Blue            0x001F      /*   0,   0, 255 */
#define Green           0x07E0      /*   0, 255,   0 */
#define Cyan            0x07FF      /*   0, 255, 255 */
#define Red             0xF800      /* 255,   0,   0 */
#define Magenta         0xF81F      /* 255,   0, 255 */
#define Yellow          0xFFE0      /* 255, 255,   0 */
#define White           0xFFFF      /* 255, 255, 255 */
#define Orange          0xFD20      /* 255, 165,   0 */
#define GreenYellow     0xAFE5      /* 173, 255,  47 */
#define Pink            0xF81F

#define ShortPressTime 100
//#define LongPressTime  500
#define NumButtons 4

long StripColors[] = {0x0000FF,0x0000FF,0x0000FF,0xFF0000,0xFF0000,0x00FF00,0x00FF00,0x00FF00}; //blue,blue,blue,red,red,green,green,green

struct ButtonStates {            // struct for holding states of our buttons
  int PIN;
  boolean State;
  boolean PrevState;
  long TimePressed; 
  boolean ShortPress;
};

ButtonStates Buttons[] = {
    {A5,false,false,0,false},
    {11,false,false,0,false},
    {13,false,false,0,false},
    {12,false,false,0,false},
};


struct ScreenDesc {        // the struct used for dashes' captions and their coordinates 
  byte ScreenNum;
  byte X_coor;
  byte Y_coor;
  String DashDesc;
};

const ScreenDesc ScreenDashCaptions[] =      // the struct used for dashes' captions and their coordinates
{
  {0,1,16,""},
  {1,10,10,"RPM"},
  {1,10,90,"Speed, km/h"},
  {1,200,10,"Gear"},
  {1,10,170,"Fuel, L"},
  {1,190,170,"Lap"},
  {2,20,10,"Water Temp, C"},
  {2,170,10,"Oil Temp, C"},
  {2,20,110,"Fuel Remaining, %"},
  {2,170,110,"Fuel Remaining, L"},
//  {3,10,10,"Last Lap"},
//  {3,250,10,"Gear"},
//  {3,10,170,"Delta time to best lap"},
//  {3,10,90,"Best lap"},
//  {3,210,170,"Lap"},
  {4,30,20,"RPM Min THR"},
  {4,30,40,"RPM Max THR"},
  {4,30,60,"RPM Autotune"},
  {4,30,80,"Water THR"},
  {4,30,100,"Oil THR"},
  {4,30,120,"Fuel % THR"},
  {4,30,140,"Fuel L THR"},
};
  
const byte NumScreens = 2;
unsigned int VarLimits[] = {4000,6000,1,100,100,10,2};    // initial thresholds
const byte SettingsFirstElement = 10; //10 - first element for settings screen in struct ScreenDashCaptions


struct FontDescriptor
{
	uint8_t	width;		// width in bits
	uint8_t	height; 	// char height in bits
	uint16_t offset;	// offset of char into char array
};

const uint8_t swis721BlkCnBT_24ptBitmaps[] =          //big font for gears
{
        0x20, 0x7F,
	0x03, 0xC0, 0x03, 0xC0, 0x07, 0xC0, 0x1F, 0xC0, 0xFF, 0xC0, 0xFF, 0xC0, 0xFF, 0xC0, 0xFF, 0xC0, 0x07, 0xC0, 0x07, 0xC0, 0x07, 0xC0, 0x07, 0xC0, 0x07, 0xC0, 0x07, 0xC0, 0x07, 0xC0, 0x07, 0xC0, 0x07, 0xC0, 0x07, 0xC0, 0x07, 0xC0, 0x07, 0xC0, 0x07, 0xC0, 0x07, 0xC0, 0x07, 0xC0, 0x07, 0xC0, 
	0x0F, 0xC0, 0x1F, 0xF0, 0x3F, 0xF8, 0x7F, 0xF8, 0x7C, 0xFC, 0xF8, 0x7C, 0xF8, 0x7C, 0xF8, 0x7C, 0xF8, 0xFC, 0x00, 0xF8, 0x01, 0xF8, 0x03, 0xF0, 0x07, 0xF0, 0x0F, 0xE0, 0x1F, 0xC0, 0x1F, 0x80, 0x3F, 0x00, 0x7E, 0x00, 0x7C, 0x00, 0x7F, 0xFC, 0xFF, 0xFC, 0xFF, 0xFC, 0xFF, 0xFC, 0xFF, 0xFC, 
	0x0F, 0xC0, 0x3F, 0xF0, 0x7F, 0xF8, 0x7F, 0xFC, 0xFC, 0xFC, 0xF8, 0x7C, 0xF8, 0x7C, 0xF8, 0x7C, 0x00, 0xF8, 0x0F, 0xF8, 0x0F, 0xF0, 0x0F, 0xF8, 0x0F, 0xFC, 0x00, 0xFC, 0x00, 0x3C, 0x00, 0x3C, 0xF8, 0x3C, 0xF8, 0x3C, 0xF8, 0x3C, 0xFC, 0x7C, 0x7F, 0xFC, 0x7F, 0xF8, 0x3F, 0xF0, 0x0F, 0xE0, 
	0x01, 0xF8, 0x01, 0xF8, 0x03, 0xF8, 0x03, 0xF8, 0x07, 0xF8, 0x07, 0xF8, 0x0F, 0x78, 0x0F, 0x78, 0x1E, 0x78, 0x1E, 0x78, 0x3C, 0x78, 0x3C, 0x78, 0x78, 0x78, 0x78, 0x78, 0xF0, 0x78, 0xFF, 0xFE, 0xFF, 0xFE, 0xFF, 0xFE, 0xFF, 0xFE, 0x00, 0xF8, 0x00, 0xF8, 0x00, 0xF8, 0x00, 0xF8, 0x00, 0xF8, 
	0xFF, 0xF8, 0xFF, 0xF8, 0xFF, 0xF8, 0xFF, 0xF8, 0xF0, 0x00, 0xF0, 0x00, 0xF0, 0x00, 0xF0, 0x00, 0xF7, 0xE0, 0xFF, 0xF0, 0xFF, 0xF8, 0xFF, 0xF8, 0xF8, 0xFC, 0x00, 0x7C, 0x00, 0x7C, 0x00, 0x7C, 0xF8, 0x7C, 0xF8, 0x7C, 0xF8, 0x7C, 0xFC, 0xFC, 0x7F, 0xF8, 0x7F, 0xF8, 0x3F, 0xF0, 0x0F, 0xC0, 
	0x0F, 0xC0, 0x3F, 0xF0, 0x7F, 0xF8, 0x7F, 0xFC, 0x7C, 0x7C, 0xF8, 0x7C, 0xF8, 0x00, 0xF8, 0x00, 0xFB, 0xE0, 0xFF, 0xF0, 0xFF, 0xF8, 0xFF, 0xF8, 0xFC, 0xFC, 0xF8, 0x7C, 0xF8, 0x7C, 0xF8, 0x7C, 0xF8, 0x7C, 0xF8, 0x7C, 0xF8, 0x7C, 0x7C, 0xF8, 0x7F, 0xF8, 0x7F, 0xF8, 0x3F, 0xF0, 0x0F, 0xC0, 
	0xFF, 0xFC, 0xFF, 0xFC, 0xFF, 0xFC, 0xFF, 0xFC, 0xFF, 0xFC, 0x00, 0x78, 0x00, 0xF8, 0x00, 0xF0, 0x01, 0xF0, 0x01, 0xF0, 0x03, 0xE0, 0x03, 0xE0, 0x03, 0xE0, 0x07, 0xC0, 0x07, 0xC0, 0x07, 0xC0, 0x0F, 0xC0, 0x0F, 0x80, 0x0F, 0x80, 0x0F, 0x80, 0x1F, 0x80, 0x1F, 0x80, 0x1F, 0x00, 0x1F, 0x00, 
	0xFC, 0x3E, 0xFC, 0x3E, 0xFE, 0x3E, 0xFE, 0x3E, 0xFE, 0x3E, 0xFF, 0x3E, 0xFF, 0x3E, 0xFF, 0x3E, 0xFF, 0xBE, 0xF7, 0xBE, 0xFF, 0xBE, 0xFF, 0xBE, 0xFB, 0xFE, 0xFB, 0xFE, 0xFB, 0xDE, 0xFB, 0xFE, 0xF9, 0xFE, 0xF9, 0xFE, 0xF9, 0xFE, 0xF8, 0xFE, 0xF8, 0xFE, 0xF8, 0xFE, 0xF8, 0x7E, 0xF8, 0x7E, 
	0xFF, 0xF0, 0xFF, 0xF8, 0xFF, 0xFC, 0xFF, 0xFE, 0xFF, 0xFE, 0xFC, 0x7E, 0xFC, 0x7E, 0xFC, 0x7E, 0xFC, 0xFE, 0xFF, 0xFC, 0xFF, 0xF8, 0xFF, 0xF0, 0xFF, 0xFC, 0xFF, 0xFC, 0xFC, 0xFE, 0xFC, 0x7E, 0xFC, 0x7E, 0xFC, 0x7E, 0xFC, 0x7E, 0xFC, 0x7E, 0xFC, 0x7E, 0xFC, 0x7E, 0xFC, 0x7E, 0xFC, 0x7E, 
};

const FontDescriptor swis721BlkCnBT_24ptDescriptors[] = 
{
	{10, 24, 0}, 		// 1 
	{14, 24, 48}, 		// 2 
	{15, 24, 96}, 		// 3 
	{15, 24, 144}, 		// 4 
	{14, 24, 192}, 		// 5 
	{14, 24, 240}, 		// 6 
	{14, 24, 288}, 		// 7 
	{15, 24, 336}, 		// N 
	{16, 24, 384}, 		// R 
};


//  const uint8_t* fontData;
//  const FontDescriptor* fontDesc;
  uint8_t textsize, rotation, font;
  
const uint8_t* fontData = swis721BlkCnBT_24ptBitmaps; 
const FontDescriptor* fontDesc = swis721BlkCnBT_24ptDescriptors; 
uint8_t fontStart = fontData[FONT_START];

  
 
Adafruit_TFTLCD tft(LCD_CS, LCD_CD, LCD_WR, LCD_RD, LCD_RESET);
// If using the shield, all control and data lines are fixed, and
// a simpler declaration can optionally be used:
// Adafruit_TFTLCD tft;
int i;
int Speed=1, NewSpeed;      // holds the speed data 
int Gear=1, NewGear;          // holds gear value data 
int Rpm=1, NewRpm;           // holds the rpm data 
float Oil=1, NewOil;           // holds the oil temp data 
float Water=1, NewWater;         // holds the water temp data 
float Fuel=1, NewFuel;        // holds the fuel data in %
float FuelL=1, MaxFuel=0, NewFuelL;      // holds the fuel data in litres
float BLap=1, NewBLap;        // holds delta to best lap
float OLap=1, NewOLap;        // holds delta to optimal lap 
long Lap=1, NewLap;           // holds the lap data (0-65535 size)
long LastLap, NewLastLap;      // holds last lap time
long BestLap, NewBestLap;      // holds best lap time
long TelRPM;
long EngineWarn;
float FuelRate, NewFuelRate;

        
int rpmleds=0;          // holds the 8 leds values
unsigned int rpmmax = 5000;    // retrieves from rpm live with minimum setting on initialization and Shutdown - Output
boolean Leds_up=false, flag=false;
unsigned long CurrentMillis, PreviousMillis=0, DataTime=0;
byte Brightness=30;          // initial brightness for LED strip
long Blink_Interval=50;      // defines blink frequency for LED strip
byte ActiveScreen;
boolean TelDrivenShift = false;

byte bufferArray[5]={0,0,0,0,0};

boolean SettingsMode = false;
byte SettingsIndex = -1;

char res[9]={' ',' ',':',' ',' ','.',' ',' ','\0'};
char WarningsDesc[]={'W','F','O','E','P','R'};

void drawChar(int16_t x, int16_t y, unsigned char c, uint16_t color, uint16_t bg, uint8_t size);

void setup(void) {
  Serial.begin(57600);
  tft.reset();
  uint16_t identifier = 0x9341;
  tft.begin(identifier);
  tft.setRotation(1);
  ActiveScreen = 1; //EEPROM.read(0);
  ShowScreen(1);
  
  if (EEPROM.read(1) != 255) {   
    EEPROM.get(1, VarLimits);
  }
 
  strip.begin();
  if (EEPROM.read(0) != 255) {   
    EEPROM.get(0, Brightness);
  }
  strip.setBrightness(Brightness);
  strip.show();
  
  for (byte i=0;i<NumButtons;i++) {
    pinMode(Buttons[i].PIN, INPUT_PULLUP);
  }
//
pinMode(13, INPUT);
//  digitalWrite(A5, HIGH);
}

void loop(void) {
  
CheckButtons();
HandleButtons();
 
  if (Serial.available() > 4)  {    
        flag=true;
        DataTime=millis();
         for (int i=0; i<5; i++) {       // read and label each byte.
          bufferArray[i] = Serial.read();
          }
  }
    else {
    if (millis()-DataTime > 5000) {
     if (flag){
     ShowFullStrip(0);
     ShowScreen(ActiveScreen);
     }
    DataTime=millis(); flag=false;
    }
  }
  
if (!SettingsMode) ParseNDraw();

}
  

void HandleButtons(void) {
 for (byte i=0;i<NumButtons;i++) {
   switch (i) {
     case 0: 
            if (Buttons[i].ShortPress && !SettingsMode) {
              ActiveScreen++;
              if (ActiveScreen > NumScreens) ActiveScreen=1;
            //  ClearAllDashes(); //clear all for drawing new screen
              ShowScreen(ActiveScreen);
            }
            break;
       case 1:
              if (Buttons[i].ShortPress) {
                SettingsMode = true;
                SettingsIndex++;
                if (SettingsIndex > 6) {
                  SettingsIndex=-1;
                  SettingsMode = false;
                  EEPROM.put(1, VarLimits);    
                  ShowScreen(ActiveScreen);
                }   
                if (SettingsMode) {
                  ShowScreen(4);
                  tft.fillRect(10,17+(20*SettingsIndex),200,21,DarkGrey);
                  PrepareDisplayElement(30,20+20*SettingsIndex,2,Black);
                  tft.print(ScreenDashCaptions[SettingsFirstElement+SettingsIndex].DashDesc);  
                  DrawCurrLimits();
                }
              }
              break;
       case 2:
              if (Buttons[i].ShortPress && SettingsMode) {
               switch (SettingsIndex) {
                 case 0:
                 case 1:
                        VarLimits[SettingsIndex]+=10;
                        break;
                 case 2:
                        ChangeRPMAutotune();
                        break;
                 default:
                        VarLimits[SettingsIndex]++;
                        break;
                }
                DrawCurrLimits();
             }
             if (Buttons[i].ShortPress && !SettingsMode) {
               if (Brightness<250) Brightness+=10;
                 BlinkStrip(); 
               }
              break;
        case 3:
               if (Buttons[i].ShortPress && SettingsMode) {
               switch (SettingsIndex) {
                 case 0:
                 case 1:
                        VarLimits[SettingsIndex]-=10;
                        break;
                 case 2:
                        ChangeRPMAutotune();
                        break;
                 default:
                        VarLimits[SettingsIndex]--;
                        break;
                }
                DrawCurrLimits();
             }
             if (Buttons[i].ShortPress && !SettingsMode) {
               if (Brightness>9) Brightness-=10;  
               BlinkStrip();
             }
              break;
   }

  }
}

void ClearAllDashes(void) {
Gear=1; Fuel=1; Lap=999;Oil=1; Water=1; Fuel=1; FuelL=1; Speed=1; Rpm=1;
}

void BlinkStrip(void) {
  strip.setBrightness(Brightness);
  ShowFullStrip(White);
  delay(200);
  ShowFullStrip(0);
  EEPROM.put(0, Brightness);
}

void ChangeRPMAutotune(void) {
 if (VarLimits[SettingsIndex] == 0) VarLimits[SettingsIndex] = 1;
   else VarLimits[SettingsIndex] = 0;
}

void DrawCurrLimits(void) {
  tft.fillRect(240,17+(20*SettingsIndex),65,21,DarkGrey);
  tft.setCursor(245,20+20*SettingsIndex);
  tft.print(VarLimits[SettingsIndex]); 
}


void CheckButtons(void) {
 long CheckTime;
 
 for (byte i=0;i<NumButtons;i++) {
  if (digitalRead(Buttons[i].PIN) == LOW) Buttons[i].State = true;
   else {
     Buttons[i].State = false;
     Buttons[i].ShortPress = false;
   }
   
  if (Buttons[i].PrevState == false && Buttons[i].State == false) {
    Buttons[i].ShortPress = false;
 //   Buttons[i].TimePressed = millis();
  }
  
  if (Buttons[i].PrevState == false && Buttons[i].State == true) {      // Pressed for 1st time, so we just remember press time
    Buttons[i].TimePressed = millis();
   }
 
  CheckTime = millis() - Buttons[i].TimePressed;
   
  if (Buttons[i].PrevState == true && Buttons[i].State == true) {
   if (CheckTime > ShortPressTime) {      // Button state changed from pressed to released
      Buttons[i].ShortPress = true;
      Buttons[i].TimePressed = millis();
    } 
    else Buttons[i].ShortPress = false; 
  }
 
  if (Buttons[i].State != Buttons[i].PrevState) {
    Buttons[i].PrevState = Buttons[i].State;
  }
 }
}
 
void ParseNDraw(void) {   
         int clr;
         
         if (bufferArray[0] == 'X') {
           TelDrivenShift = true;
           TelRPM = NormalizeData(bufferArray);
           rpmleds = map(TelRPM,5,60,8,-1);      // for ShiftIndicatorPct we start to light leds on 5% and begin to flash on 60%
         }
         
         if (bufferArray[0] == 'R') {
           NewRpm = (int)NormalizeData(bufferArray);
           if (Rpm != NewRpm) {
              if (ActiveScreen==1) DisplayElement(20,30,5,Rpm,NewRpm,White,Black);
              Rpm = NewRpm;
              }
           if (!TelDrivenShift) {             // if not telemetry driven shift point, than construct stripe and display it
             if (VarLimits[2] == 1) {   
               if (Rpm>rpmmax) rpmmax=Rpm;
               rpmleds = map(NewRpm,0,rpmmax*0.98,8,-1);
             }
            else rpmleds = map(NewRpm,VarLimits[0],VarLimits[1]*0.99,8,0);
           }
            DisplayLeds(rpmleds);
         }

         if (bufferArray[0] == 'E') {
           EngineWarn = NormalizeData(bufferArray);
           for (byte i=0;i<6;i++) {
             if (EngineWarn == (0x01 << i)) {
               tft.fillRect(280,130,30,30,Black);
               tft.setCursor(280,130);
               tft.setTextColor(Red);
               tft.print(WarningsDesc[i]);
             }
             if (EngineWarn == 0) tft.fillRect(280,130,30,40,Black);
            }
          }
           
         
           
         if (ActiveScreen == 1) {
         switch (bufferArray[0]) {
           case 'G':
                 ShowGearDash(200,40);
                 break;
             case 'S':
                  NewSpeed = (int)NormalizeData(bufferArray);
                  if (Speed != NewSpeed) {
                   DisplayElement(20,110,5,Speed,NewSpeed,White,Black);
                    Speed = NewSpeed;
                  }
                  break;
//             case 'F':
//                  ShowFuelDash(20,190);
             case 'U':
                  ShowFuelLitrDash(10,190);
                  break;
             case 'L':
                  ShowLapDash(180,190);
                  break;
      }
    }
      if (ActiveScreen == 2) {
        switch (bufferArray[0]) {
           case 'O':
            NewOil = (float)NormalizeData(bufferArray)/10;
           //  NewOil = NormalizeData(bufferArray)/10;
                  if (Oil != NewOil) {
                    if (NewOil > VarLimits[4]) {
                      clr = Red;
                    }
                    else if (NewOil > VarLimits[4]*0.8) {
                      clr = Yellow;
                    }
                    else clr = White;
                    DisplayElementf(170,30,5,Oil,NewOil,clr,Black,1);
                    Oil = NewOil;
                  }
                  break;
           case 'W':
             NewWater = (float)NormalizeData(bufferArray)/10;
                  if (Water != NewWater) {
                    if (NewWater > VarLimits[3]) {
                      clr = Red;
                    }
                    else if (NewWater > VarLimits[3]*0.8) {
                      clr = Yellow;
                    }
                    else clr = White;
                    DisplayElementf(20,30,5,Water,NewWater,clr,Black,1);
                    Water = NewWater;
                  }
                  break;
           case 'F':
                  ShowFuelDash(20,130);
                  break;
           case 'U':
                  ShowFuelLitrDash(170,130);
                  break;
        }
      }
//      if (ActiveScreen == 3) {
//        switch (bufferArray[0]) {
//           case 'A':
//             NewLastLap = NormalizeData(bufferArray);
//                  if (LastLap != NewLastLap) {
//                    ConvFl2Cl(LastLap);
//                    PrepareDisplayElement(10,30,4,Black);
//                    tft.print(res);
//                    ConvFl2Cl(NewLastLap);
//                    PrepareDisplayElement(10,30,4,White);
//                    tft.print(res);
//                    LastLap = NewLastLap;
//                  }
//                  break;
//           case 'T':
//             NewBestLap = NormalizeData(bufferArray);
//                  if (BestLap != NewBestLap) {
//                    ConvFl2Cl(BestLap);
//                    PrepareDisplayElement(10,110,4,Black);
//                    tft.print(res);
//                    ConvFl2Cl(NewBestLap);
//                    PrepareDisplayElement(10,110,4,White);
//                    tft.print(res);
//                    BestLap = NewBestLap;
//                  }
//                  break;
//           case 'B':
//             NewBLap = (float)NormalizeData(bufferArray)/100;
//                  if (BLap != NewBLap) {
//                    if (NewBLap < 0) clr = Green;
//                     else clr = Red;
//                  DisplayElementf(10,190,5,BLap,NewBLap,clr,Black,2); 
//                   BLap = NewBLap;
//                  }
//                  break;
//           case 'G':
//                 ShowGearDash(250,40);
//                 break;
//           case 'L':
//                 ShowLapDash(210,190);
//                 break;
//        }
//      }
  }

void ShowFuelLitrDash(int x, int y) {
   int clr;
   NewFuelL = (float)NormalizeData(bufferArray)/10;
   if (MaxFuel < NewFuelL) MaxFuel = NewFuelL;   //store max fuelL
   if (FuelL != NewFuelL) {
    if (NewFuelL > VarLimits[6]*2) {
       clr = Green;
     }
     else if (NewFuelL <= VarLimits[6]) {
       clr = Red;
     }
     else clr = Yellow;
     DisplayElementf(x,y,5,FuelL,NewFuelL,clr,Black,1);
     FuelL = NewFuelL;
   }
}


void ShowLapDash(int x, int y) {
  NewLap = NormalizeData(bufferArray);
  if (Lap != NewLap) {
    DisplayElement(x,y,5,Lap,NewLap,White,Black);
    if (NewLap > 1 && FuelL != 1) {
      NewFuelRate = (float)(MaxFuel - FuelL)/NewLap;
      DisplayElementf(x+75,y+15,3,FuelRate,NewFuelRate,White,Black,1);
      FuelRate = NewFuelRate;
    }
    Lap = NewLap;
   }
}
                  
void ShowFuelDash(int x, int y) {
int clr;
  NewFuel = (float)NormalizeData(bufferArray)/100;
  if (Fuel != NewFuel) {
   if (NewFuel > VarLimits[5]*2) {
     clr = Green;
   }
   else if (NewFuel <= VarLimits[5]) {  
     clr = Red;
   }
   else clr = Yellow;
  DisplayElementf(x,y,5,Fuel,NewFuel,clr,Black,1);
  Fuel = NewFuel;
   }
}

void ShowGearDash(int x, int y) {
  NewGear = (int)NormalizeData(bufferArray);
  if (Gear != NewGear) {
   switch (Gear) {
   case 0:
   case 2: 
       tft.fillRect(200,40,232,122,Black); break;
               }
   if (NewGear > 0 and NewGear < 8) drawChar(x,y,NewGear-1,Green,Black,3);
   if (NewGear == 0) drawChar(x,y,7,Green,Black,3);
   if (NewGear == 255) drawChar(x,y,8,Green,Black,3);
   Gear = NewGear;
                 }
}

/*void ConvFl2Cl(long digit) {      // convert long to string like mm:ss.xx
  int tmp1,tmp2,tmp3;//,bigM,smallM;
//  String str;
//  char res[8]={' ',' ',':',' ',' ','.',' ',' '};
//  tft.println(digit);
  tmp1 = (int)digit/100;
  tmp2 = (int)tmp1/60;
  tmp3 = (int)tmp2/10;
//  tft.print(tmp3);
//  smallM = (int)tmp2%10;
  res[0] = char(tmp3)+48;
//  tft.print(res[0]);
  tmp3 = (int)tmp2%10;
  res[1] = char(tmp3)+48;
//  str = (String)tmp2 + ":";
 // tmp1 = (int)digit/100;
  tmp2 = (int)tmp1%60;
  tmp3 = (int)tmp2/10;
  res[3] = char(tmp3)+48;
  tmp3 = (int)tmp2%10;
  res[4] = char(tmp3)+48;
//  str = str + (String)tmp2 + ".";
  tmp2 = (int)digit%100;
  tmp3 = (int)tmp2/10;
  res[6] = char(tmp3)+48;
  tmp3 = (int)tmp2%10;
  res[7] = char(tmp3)+48;  
}*/

void ShowScreen(byte ScreenNumber) {
  byte size = 1;
  if (ScreenNumber == 4) size=2;
  ClearAllDashes();
  tft.fillScreen(Black);
  PrepareDisplayElement(0,0,size,DarkGrey);
  for (int i=1; i<ScreenDashCaptions[0].Y_coor; i++) {
    if (ScreenDashCaptions[i].ScreenNum == ScreenNumber) {
      tft.setCursor(ScreenDashCaptions[i].X_coor,ScreenDashCaptions[i].Y_coor);
      tft.print(ScreenDashCaptions[i].DashDesc);
    }
  }
}


void ShowFullStrip(uint32_t color) {
    for(uint16_t i=0; i<strip.numPixels(); i++) {
      strip.setPixelColor(i, color);
      strip.show();
   }
}

void DisplayLeds(int mode) {
    if (mode < 0) {
      CurrentMillis=millis();
      if (CurrentMillis - PreviousMillis > Blink_Interval) {
         PreviousMillis=CurrentMillis;
         Leds_up=Leds_up^1;
         if (Leds_up) ShowFullStrip(Black);
           else ShowFullStrip(White);
       }
     }
     
     if (mode >= 0 && mode <= 7) {
       ShowFullStrip(Black);
       for(byte i=mode; i<8; i++) {
         strip.setPixelColor(i, StripColors[i]);
       }
       strip.show();
     }
     
     if (mode > 7) ShowFullStrip(Black);
 }   

void PrepareDisplayElement(int x, int y, byte size, int color) {
      tft.setCursor(x, y);
      tft.setTextColor(color);
      tft.setTextSize(size);
}  

void DisplayElement(int x, int y, byte size, int dash, int newdash, int color, int bgcolor) {
      PrepareDisplayElement(x,y,size,bgcolor);
      tft.print(dash);
      PrepareDisplayElement(x,y,size,color);
      tft.print(newdash);
}

void DisplayElementf(int x, int y, byte size, float dash, float newdash, int color, int bgcolor, byte digits) {
      PrepareDisplayElement(x,y,size,bgcolor);
      tft.print(dash,digits);
      PrepareDisplayElement(x,y,size,color);
      tft.print(newdash,digits);
}

long NormalizeData(byte x[5]) {
        
        unsigned long tmp=0;
        unsigned long result=0;

    for (int i=1; i<5; i++) {
          if (x[i]>47 && x[i]<58 ) {   //for xA to xF
            x[i]=x[i]-48;
          }                       
          if (  x[i]>64 && x[i]<71 ) { //for x0 to x9
            x[i]=x[i]-55;               
          }
          tmp = (tmp*16) + x[i];
    //    result = result + x[i] * pow(16,4-i); 
        }    
    //    if (tmp > 2147483647) result = tmp - 4294967292;
    //     else result = tmp;
    result = tmp;
        return result;
}
             
        
void drawChar(int16_t x, int16_t y, unsigned char c,
			    uint16_t color, uint16_t bg, uint8_t size) {
  uint8_t bitCount=0;
  uint16_t fontIndex = fontDesc[c].offset + 2; //((fontDesc + c)->offset) + 2;
  for (int8_t i=0; i<fontDesc[c].height; i++ ) {	// i<fontHeight
    uint8_t line;
    for (int8_t j = 0; j<fontDesc[c].width; j++) {			//j<fontWidth
      if (bitCount++%8 == 0) {
        line = fontData[fontIndex]; fontIndex++;
      }
      if (line & 0x80) {
 //       if (size == 1) {// default sizeFast
 //         tft.drawPixel(x+j, y+i, color);
 //         }
 //       else {  // big size
          tft.fillRect(x+(j*size), y+(i*size), size, size, color);
 //       } 
      } else if (bg != color) {
 //       if (size == 1) // default size
 //         tft.drawPixel(x+j, y+i, bg);
 //       else {  // big size
          tft.fillRect(x+j*size, y+i*size, size, size, bg);
 //       }
      }
      line <<= 1;
    }
    bitCount = 0;
  }
}


